﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.Model;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfo("Split categorial column", 1, ColumnGroupState.ColumnSelected, "Column Commands",
    Position = 16, SelectedColumns = 1)]
  public class SplitCategorialColumnCommand : ColumnCommandBase {
    private int addedColumnsCount;
    private const double SETVALUE = 1.0;
    private const double NOTSETVALUE = 0.0;

    private SplitCategorialColumnCommand()
      : base(null, string.Empty, -1) {
    }

    public SplitCategorialColumnCommand(DataSet dataSet, string columnGroupName, int columnIndex)
      : base(dataSet, columnGroupName, columnIndex) {
    }

    public override string Description {
      get { return "Split categorial column"; }
    }

    public override void Execute() {
      base.Execute();
      if (Column.NullValuesCount != 0)
        throw new CommandExecutionException("Categorial column must not contain null values.", this);

      int columnIndex = this.ColumnGroup.IndexOfColumn(Column);
      Dictionary<string, DoubleColumn> addedColumns = new Dictionary<string, DoubleColumn>();
      foreach (string value in Column.ValuesEnumerable.Cast<IComparable>().Distinct().OrderBy(c => c).Select(o => o.ToString())) {
        DoubleColumn doubleColumn = new DoubleColumn(Column.Name + " " + value.ToString());
        addedColumns[value] = doubleColumn;
        ColumnGroup.InsertColumn(columnIndex + addedColumnsCount + 1, doubleColumn);
        addedColumnsCount++;
      }

      for (int row = 0; row < ColumnGroup.RowCount; row++) {
        string value = Column.GetValue(row).ToString();
        foreach (KeyValuePair<string, DoubleColumn> pair in addedColumns) {
          if (pair.Key == value)
            pair.Value.AddValue(SETVALUE);
          else
            pair.Value.AddValue(NOTSETVALUE);
        }
      }
      ColumnGroup.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      int columnIndex = this.ColumnGroup.IndexOfColumn(Column);
      for (int i = 0; i < addedColumnsCount; i++)
        this.ColumnGroup.RemoveColumn(columnIndex + 1);
      addedColumnsCount = 0;
      ColumnGroup.FireChanged();
    }
  }
}
